"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint max-depth: ["error", 5] */
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const fs = require("fs");
const path = require("path");
const SymbolTree = require("symbol-tree");
const date_fns_1 = require("date-fns");
const base_1 = require("../base");
const getCompletion_1 = require("../utils/streams/getCompletion");
const teamUtils_1 = require("../utils/misc/teamUtils");
const fetchIssues_1 = require("../utils/data/fetchIssues");
const fetchChildren_1 = require("../utils/data/fetchChildren");
const prepareIssuesData_1 = require("../utils/forecast/prepareIssuesData");
const optimizePayload_1 = require("../utils/forecast/optimizePayload");
const treeUtils_1 = require("../utils/misc/treeUtils");
const getChildren_1 = require("../utils/streams/getChildren");
const id_1 = require("../utils/misc/id");
const trimIssue = (issue) => {
    var _a, _b, _c, _d;
    return {
        key: issue.key,
        closedAt: issue.closedAt,
        points: issue.points,
        openedForDays: issue.openedForDays,
        fields: {
            summary: issue.fields.summary,
            created: issue.fields.created,
            resolution: ((_a = issue === null || issue === void 0 ? void 0 : issue.fields) === null || _a === void 0 ? void 0 : _a.resolution) !== undefined &&
                ((_b = issue === null || issue === void 0 ? void 0 : issue.fields) === null || _b === void 0 ? void 0 : _b.resolution) !== null
                ? issue.fields.resolution.name
                : undefined,
            priority: ((_c = issue === null || issue === void 0 ? void 0 : issue.fields) === null || _c === void 0 ? void 0 : _c.priority) !== undefined &&
                ((_d = issue === null || issue === void 0 ? void 0 : issue.fields) === null || _d === void 0 ? void 0 : _d.priority) !== null
                ? issue.fields.priority.name
                : undefined,
        },
    };
};
class Streams extends base_1.default {
    async run() {
        const { flags } = this.parse(Streams);
        const userConfig = this.userConfig;
        const cacheDir = path.join(this.config.configDir, 'cache');
        for (const team of userConfig.teams) {
            this.log(`Fetching queries for team: ${team.name}`);
            const fetchedIssues = [];
            const completedStreams = [];
            const allForecastIssuesKeys = [];
            const forecastStreams = [];
            for (const stream of team.streams) {
                this.log(`${team.name}/${stream.name}: Processing completion data for stream: ${stream.name}`);
                let childIssues = [];
                if (stream.completion.childOf !== undefined) {
                    this.log(`${team.name}/${stream.name}: Collecting children using JQL: ${stream.completion.childOf}`);
                    // Get list of issue keys matching the JQL query
                    childIssues = await (0, getChildren_1.getChildrenKey)(userConfig.jira, stream.completion.childOf, cacheDir, flags.useCache);
                }
                const completionDays = await (0, getCompletion_1.default)(team, userConfig.jira, stream, cacheDir, fetchedIssues, childIssues.map(i => i.key));
                // This records issues that were already accounted for completion
                // in preview streams for the team
                for (const d of completionDays) {
                    for (const i of d.issues) {
                        fetchedIssues.push(i);
                    }
                }
                completedStreams.push({
                    key: (0, id_1.getId)(stream.name),
                    name: stream.name,
                    jql: stream.completion.jql,
                    childOf: stream.completion.childOf === undefined
                        ? ''
                        : stream.completion.childOf,
                    days: completionDays.map((d) => {
                        return Object.assign(Object.assign({}, d), { issues: d.issues.map((i) => trimIssue(i)) });
                    }),
                    issues: childIssues.reduce((acc, issue) => {
                        if (issue.parent.key !== '' &&
                            acc.find(i => i.key === issue.parent.key) === undefined) {
                            acc.push({
                                key: issue.parent.key,
                                summary: issue.parent.summary,
                                issues: [issue.key],
                            });
                        }
                        else if (issue.parent.key !== '' &&
                            acc.find(i => i.key === issue.parent.key) !== undefined) {
                            return acc.map(s => {
                                if (s.key === issue.parent.key) {
                                    return Object.assign(Object.assign({}, s), { issues: [...s.issues, issue.key] });
                                }
                                return s;
                            });
                        }
                        return acc;
                    }, []),
                    // childIssues,
                });
                this.log(`${team.name}/${stream.name}: Processing forecast data for stream: ${stream.name}`);
                const forecastIssues = await (0, fetchIssues_1.default)(stream.forecast.jql, team.name, stream.name, userConfig.jira, cacheDir, flags.useCache);
                this.log(`${team.name}/${stream.name}: Number of issues fetched: ${forecastIssues.length}`);
                // Fetch all of the initiatives children and build a tree
                const issuesTree = new SymbolTree();
                const treeRoot = {};
                for (const issue of forecastIssues.filter((i) => !allForecastIssuesKeys.includes(i.key))) {
                    issuesTree.appendChild(treeRoot, issue);
                    let children = [];
                    if (stream.forecast.fetchChild === true) {
                        children = await (0, fetchChildren_1.default)(userConfig.jira, issue.key, cacheDir, flags.useCache);
                    }
                    for (const l1child of children
                        .filter((c) => !allForecastIssuesKeys.includes(c.key))
                        .filter((ic) => ic.fields[userConfig.jira.fields.parentInitiative] ===
                        issue.key)) {
                        issuesTree.appendChild(issue, l1child);
                        for (const l2child of children
                            .filter((c) => !allForecastIssuesKeys.includes(c.key))
                            .filter((ic) => ic.fields[userConfig.jira.fields.parentEpic] === l1child.key)) {
                            issuesTree.appendChild(l1child, l2child);
                        }
                    }
                    for (const c of children) {
                        if (!allForecastIssuesKeys.includes(c.key)) {
                            allForecastIssuesKeys.push(c.key);
                        }
                    }
                }
                for (const i of forecastIssues) {
                    if (!allForecastIssuesKeys.includes(i.key)) {
                        allForecastIssuesKeys.push(i.key);
                    }
                }
                cli_ux_1.default.action.start('Preparing metrics for all initiatives in the tree ');
                // Update the tree with completion and week by week metrics
                (0, prepareIssuesData_1.default)(issuesTree, treeRoot, 0, userConfig);
                cli_ux_1.default.action.stop(' done');
                const prepTree = (0, treeUtils_1.exportTree)(issuesTree, treeRoot);
                cli_ux_1.default.action.start('Optimizing payload for frontend');
                const slimIssues = (0, optimizePayload_1.default)(prepTree);
                cli_ux_1.default.action.stop(' done');
                forecastStreams.push({
                    key: (0, id_1.getId)(stream.name),
                    name: stream.name,
                    jql: stream.forecast.jql,
                    fetchChild: stream.forecast.fetchChild,
                    effortPct: stream.forecast.effortPct,
                    issues: slimIssues,
                });
            }
            this.log(`Team: ${team.name} Fetched data for ${completedStreams.length} streams`);
            const teamData = {
                name: team.name,
                from: team.from,
                excludeDays: team.excludeDays,
                jiraHost: userConfig.jira.host,
                resolutions: userConfig.jira.resolutions,
                completion: completedStreams,
                forecast: forecastStreams,
                updatedAt: new Date().toJSON(),
            };
            const fileStream = fs.createWriteStream(path.join(cacheDir, `stream-artifacts-${(0, teamUtils_1.getTeamId)(team.name)}.json`), { flags: 'w' });
            fileStream.write(JSON.stringify(teamData));
            fileStream.end();
            console.log('-----');
            // Keep one file per week to keep track of history
            const fileStreamWeek = fs.createWriteStream(path.join(cacheDir, `stream-artifacts-history-${(0, teamUtils_1.getTeamId)(team.name)}-${(0, date_fns_1.format)(new Date(), 'yyyy-ww')}.json`), { flags: 'w' });
            fileStreamWeek.write(JSON.stringify(teamData));
            fileStreamWeek.end();
            this.log('------------');
        }
    }
}
exports.default = Streams;
Streams.description = 'Fetches Completion and Forecast streams per team';
Streams.flags = Object.assign(Object.assign({}, base_1.default.flags), { help: command_1.flags.help({ char: 'h' }), useCache: command_1.flags.boolean({
        char: 'c',
        default: false,
        description: 'Use local cache instead of fetching the data from Jira',
    }) });
