"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint max-depth: ["error", 5] */
const command_1 = require("@oclif/command");
const path = require("path");
const fs = require("fs");
const date_fns_1 = require("date-fns");
const base_1 = require("../base");
const getVersions_1 = require("../utils/versions/getVersions");
const groupVersions_1 = require("../utils/versions/groupVersions");
const getIssues_1 = require("../utils/versions/getIssues");
const trimIssues_1 = require("../utils/versions/trimIssues");
const sortByNameDesc = (a, b) => {
    // Use toUpperCase() to ignore character casing
    const keyA = a.name.toUpperCase();
    const keyB = b.name.toUpperCase();
    let comparison = 0;
    if (keyA > keyB) {
        comparison = -1;
    }
    else if (keyA < keyB) {
        comparison = 1;
    }
    return comparison;
};
class Versions extends base_1.default {
    async run() {
        const userConfig = this.userConfig;
        const cacheDir = path.join(this.config.configDir, 'cache');
        // Collect raw versions from the configured projects in Jira
        let versions = [];
        for (const projectKey of userConfig.versions.projectKeys) {
            this.log(`Fetching versions for project: ${projectKey}`);
            const projectVersions = await (0, getVersions_1.default)(projectKey, userConfig.jira);
            this.log(`Fetched: ${projectVersions.length} version from project: ${projectKey}`);
            versions.push(...projectVersions);
        }
        this.log(`Fetched a total of ${versions.length} individual versions across all projets`);
        // Filter out versions with a release date prior to the one set in the config
        versions = versions.filter((v) => v.releaseDate === undefined ||
            v.releaseDate.getTime() >
                (0, date_fns_1.parse)(userConfig.versions.from, 'yyyy-MM-dd', new Date()));
        this.log(`There is a total of ${versions.length} versions released after ${userConfig.versions.from}`);
        // Regroup versions by name (issues spanning multiple projects)
        versions = (0, groupVersions_1.default)(versions);
        this.log(`There is a total of ${versions.length} unique versions (when groupped by name)`);
        // Sort the versions by name
        versions = versions.sort(sortByNameDesc);
        // Fetch all issues corresponding to these versions
        // The list of issues is cached to avoid re-fetching
        const versionsWithIssues = [];
        // An issue is only counted once,
        // this array is used to keep track of issues
        const fetchedIssues = [];
        for (const version of versions) {
            let issues = await (0, getIssues_1.default)(userConfig.jira, version, cacheDir);
            // Trim issues object to reduce the payload
            // Filter out issues that are already present
            issues = (0, trimIssues_1.default)(issues).filter(i => {
                if (fetchedIssues.includes(i.key)) {
                    return false;
                }
                fetchedIssues.push(i.key);
                return true;
            });
            versionsWithIssues.push(Object.assign(Object.assign({}, version), { issues }));
        }
        this.log(`Writing all analyzed versions to cache`);
        const fileStream = fs.createWriteStream(path.join(cacheDir, `versions.json`), { flags: 'w' });
        fileStream.write(JSON.stringify({
            versions: versionsWithIssues,
            updatedAt: new Date().toJSON(),
            jiraHost: userConfig.jira.host,
            monthsToChart: userConfig.versions.monthsToChart,
            defaultFilters: {
                name: userConfig.versions.defaultFilters.name === undefined
                    ? ''
                    : userConfig.versions.defaultFilters.name,
                projectKey: userConfig.versions.defaultFilters.projectKey === undefined
                    ? ''
                    : userConfig.versions.defaultFilters.projectKey,
                label: userConfig.versions.defaultFilters.label === undefined
                    ? ''
                    : userConfig.versions.defaultFilters.label,
                issueType: userConfig.versions.defaultFilters.issueType === undefined
                    ? ''
                    : userConfig.versions.defaultFilters.issueType,
                priority: userConfig.versions.defaultFilters.priority === undefined
                    ? ''
                    : userConfig.versions.defaultFilters.priority,
            },
        }));
        fileStream.end();
    }
}
exports.default = Versions;
Versions.description = 'Fetches details about completed versions';
Versions.flags = Object.assign(Object.assign({}, base_1.default.flags), { help: command_1.flags.help({ char: 'h' }) });
